@@ -1,5 +1,6 @@
-from hhd.controller import Axis, Button, Configuration
+from hhd.controller import Axis, Button
 from hhd.controller.physical.evdev import B, to_map
+from hhd.plugins import gen_gyro_state
 
 GPD_TOUCHPAD_BUTTON_MAP: dict[int, Button] = to_map(
     {
@@ -35,3 +36,5 @@ GPD_WIN_MAX_2_2023_MAPPINGS: dict[str, tuple[Axis, str | None, float, float | No
     "anglvel_z": ("gyro_y", "anglvel", -1, None),
     "timestamp": ("imu_ts", None, 1, None),
 }
+
+GPD_WIN_4_8840U_MAPPINGS = gen_gyro_state("z", True, "x", False, "y", True)
\ No newline at end of file
