@@ -10,8 +10,8 @@ import Xlib
 from Xlib import XK, X, Xatom, display, error
 from Xlib.ext.xtest import fake_input
 
-from hhd.plugins import Context, Emitter
-from hhd.utils import restore_priviledge, switch_priviledge
+from hhd.plugins import Emitter, Config
+from hhd.utils import save_priviledge, restore_priviledge, switch_priviledge
 
 logger = logging.getLogger(__name__)
 
@@ -112,10 +112,17 @@ class QamHandlerGamescope:
                 pass
 
 
-def find_x11_auth(ctx: Context):
+def find_x11_auth(uid: int):
     # TODO: Fix hardcoding runtime dir
-    LOCATION = f"/run/user/{ctx.euid}"
-    for fn in sorted(os.listdir(LOCATION)):
+    LOCATION = f"/run/user/{uid}"
+    
+    try:
+        files = sorted(os.listdir(LOCATION))
+    except FileNotFoundError:
+        logger.warning(f"Could not find X11 auth files in {LOCATION}.")
+        return None
+
+    for fn in files:
         if (
             # KDE
             fn.startswith("xauth_")
@@ -125,10 +132,19 @@ def find_x11_auth(ctx: Context):
             return os.path.join(LOCATION, fn)
 
 
-def find_x11_display(ctx: Context):
-    for fn in sorted(os.listdir(X11_DIR)):
-        if fn and os.stat(X11_DIR + fn).st_uid == ctx.euid:
-            return ":" + fn[1:].decode()
+def find_x11_display():
+    # First, try to find an X11 display owned by the current user
+    # If not found, try to find one owned by root
+    for skip_root in (True, False):
+        for fn in sorted(os.listdir(X11_DIR)):
+            if not fn.startswith(b"X"):
+                continue
+            uid = os.stat(X11_DIR + fn).st_uid
+            if skip_root and uid == 0:
+                continue
+            return ":" + fn[1:].decode(), uid
+
+    return None, 0
 
 
 def get_gamescope_displays():
@@ -145,7 +161,7 @@ def get_gamescope_displays():
 
         fn = ln.split(b" ")[0]
         disp = ":" + fn[len(X11_DIR) + 1 :].decode()
-        out.append(disp)
+        out.append((disp, os.stat(fn).st_uid))
     return out
 
 
@@ -153,30 +169,51 @@ def is_gamescope_running():
     return bool(get_gamescope_displays())
 
 
-def get_overlay_display(displays: Sequence[str], ctx=None):
+def get_overlay_display(displays: Sequence[tuple[str, int]]):
     """Probes the provided gamescope displays to find the overlay one."""
 
     # FIXME: Fix authentication without priviledge deescalation
-    if ctx:
-        old = switch_priviledge(ctx, False)
-    else:
-        old = None
+    old = save_priviledge()
 
     try:
-        for disp in displays:
+        for disp, uid in displays:
             try:
+                restore_priviledge(old)
+                switch_priviledge(uid)
+
                 d = display.Display(disp)
 
                 atoms = [d.get_atom_name(v) for v in d.screen().root.list_properties()]
                 if "GAMESCOPE_FOCUSED_WINDOW" in atoms:
-                    return d, disp
+                    return d, disp, uid
 
                 d.close()
-            except Exception:
-                pass
+            except Exception as e:
+                import traceback
+
+                traceback.print_exc()
     finally:
-        if old:
-            restore_priviledge(old)
+        restore_priviledge(old)
+
+
+def apply_gamescope_config(display: display.Display, config: Config, prev: dict):
+    apply = False
+
+    halfhz = config.get("steamui_halfhz", None)
+    halfhz_rev = prev.get("steamui_halfhz", None)
+    if halfhz is not None and halfhz != halfhz_rev:
+        display.screen().root.change_property(
+            display.get_atom("GAMESCOPE_STEAMUI_HALFHZ"),
+            Xatom.CARDINAL,
+            32,
+            [int(halfhz)],
+        )
+        logger.info(f"Setting SteamUI halfhz to {halfhz}.")
+        prev["steamui_halfhz"] = halfhz
+        apply = True
+
+    if apply:
+        display.flush()
 
 
 def find_wins(display: display.Display, win: list[str], atoms: list[str] = []):
@@ -293,6 +330,12 @@ def prepare_hhd(display, hhd, steam=None):
         hhd.change_property(
             display.get_atom("STEAM_GAME"), Xatom.CARDINAL, 32, [HHD_ID]
         )
+        # If steam is missing, gamescope disables window controls, so we have
+        # to play with the opacity to hide.
+        hhd.change_property(
+            display.get_atom("_NET_WM_WINDOW_OPACITY"), Xatom.CARDINAL, 32, [0]
+        )
+
     hhd.change_property(display.get_atom("STEAM_NOTIFICATION"), Xatom.CARDINAL, 32, [0])
     hhd.change_property(display.get_atom("STEAM_BIGPICTURE"), Xatom.CARDINAL, 32, [1])
     hhd.change_property(display.get_atom("GAMESCOPE_NO_FOCUS"), Xatom.CARDINAL, 32, [1])
@@ -313,6 +356,20 @@ def process_events(disp):
     return True
 
 
+def get_current_game(display):
+    stat_game = display.get_atom("GAMESCOPE_FOCUSED_APP_GFX")
+    game = display.screen().root.get_property(stat_game, Xatom.CARDINAL, 0, 15)
+    return game.value[0] if game and game.value else None
+
+
+def set_dpms(display, enable: bool):
+    stat_dpms = display.get_atom("GAMESCOPE_DPMS")
+    display.screen().root.change_property(
+        stat_dpms, Xatom.CARDINAL, 32, [1 if enable else 0]
+    )
+    display.flush()
+
+
 def update_steam_values(display, steam, old: CachedValues | None):
     stat_focus = display.get_atom("STEAM_INPUT_FOCUS")
     stat_overlay = display.get_atom("STEAM_OVERLAY")
@@ -388,6 +445,11 @@ def show_hhd(display, hhd, steam):
         # logger.info(f"Setting HHD as game '{new_id}' to disable steam navigation.")
         # hhd.change_property(stat_game, Xatom.CARDINAL, 32, [new_id])
 
+    if not steam:
+        hhd.change_property(
+            display.get_atom("_NET_WM_WINDOW_OPACITY"), Xatom.CARDINAL, 32, [0xFFFFFFFF]
+        )
+
     if touch_was_set:
         # Give it a bit of time before setting the touch target to avoid steam
         # messing with it
@@ -423,10 +485,23 @@ def hide_hhd(display, hhd, steam, old: CachedValues | None):
                 stat_click, Xatom.CARDINAL, 32, [old.touch]
             )
 
+    if not steam:
+        hhd.change_property(
+            display.get_atom("_NET_WM_WINDOW_OPACITY"), Xatom.CARDINAL, 32, [0x0]
+        )
+
     display.flush()
     display.sync()
 
 
+def find_focusable_windows(display):
+    stat_focusable = display.get_atom("GAMESCOPE_FOCUSABLE_APPS")
+    focusable = display.screen().root.get_property(
+        stat_focusable, Xatom.CARDINAL, 0, 50
+    )
+    return focusable.value if focusable and focusable.value else []
+
+
 def make_hhd_not_focusable(display):
     stat_focused = display.get_atom("GAMESCOPECTRL_BASELAYER_APPID")
     stat_focusable = display.get_atom("GAMESCOPE_FOCUSABLE_APPS")
@@ -481,12 +556,12 @@ def monitor_gamescope(emit: Emitter, ctx, should_exit: TEvent):
     while not should_exit.is_set():
         # Wait for gamescope
         try:
-            res = get_overlay_display(get_gamescope_displays(), ctx)
+            res = get_overlay_display(get_gamescope_displays())
             if not res:
                 time.sleep(GAMESCOPE_WAIT)
                 continue
 
-            d, name = res
+            d, name, _ = res
             logger.info(f"Found gamescope display {name}")
             r = d.screen().root
             r.change_attributes(event_mask=X.PropertyChangeMask)
