@@ -11,6 +11,7 @@ import evdev
 from hhd.utils import Context, is_steam_gamepad_running, run_steam_command
 
 from .const import PowerButtonConfig
+from ..power.power import supports_sleep, emergency_hibernate
 
 logger = logging.getLogger(__name__)
 
@@ -65,43 +66,59 @@ def register_hold_button(b: PowerButtonConfig) -> evdev.InputDevice | None:
     return None
 
 
-def run_steam_shortpress(perms: Context):
-    return run_steam_command("steam://shortpowerpress", perms)
+_supports_sleep = None
 
 
-def run_steam_longpress(perms: Context):
-    return run_steam_command("steam://longpowerpress", perms)
+def run_steam_shortpress():
+    global _supports_sleep
+    # FIXME: This should be patched in systemd instead
+    if _supports_sleep is None:
+        _supports_sleep = supports_sleep()
+
+    if _supports_sleep:
+        return run_steam_command("steam://shortpowerpress")
+    else:
+        emergency_hibernate(shutdown=False)
+        return True
 
 
-def power_button_run(cfg: PowerButtonConfig, ctx: Context, should_exit: Event):
+def run_steam_longpress():
+    return run_steam_command("steam://longpowerpress")
+
+
+def power_button_run(cfg: PowerButtonConfig, should_exit: Event, emit):
     match cfg.type:
         case "only_press":
             logger.info(
                 f"Starting multi-device powerbutton handler for device '{cfg.device}'."
             )
-            power_button_multidev(cfg, ctx, should_exit)
+            power_button_multidev(cfg, should_exit, emit)
         case "hold_emitted":
             logger.info(
                 f"Starting timer based powerbutton handler for device '{cfg.device}'."
             )
-            power_button_timer(cfg, ctx, should_exit)
+            power_button_timer(cfg, should_exit, emit)
         case "hold_isa":
             logger.info(
                 f"Starting isa keyboard powerbutton handler for device '{cfg.device}'."
             )
-            power_button_isa(cfg, ctx, should_exit)
+            power_button_isa(cfg, should_exit, emit)
+        case "disabled":
+            logger.info(
+                f"Power button plugin disabled for device '{cfg.device}', not starting."
+            )
         case _:
             logger.error(f"Invalid type in config '{cfg.type}'. Exiting.")
 
 
-def power_button_isa(cfg: PowerButtonConfig, perms: Context, should_exit: Event):
+def power_button_isa(cfg: PowerButtonConfig, should_exit: Event, emit):
     press_dev = None
     press_devs = []
     hold_dev = None
     try:
         while not should_exit.is_set():
             # Initial check for steam
-            if not is_steam_gamepad_running(perms):
+            if not is_steam_gamepad_running():
                 # Close devices
                 if press_devs:
                     for d in press_devs:
@@ -114,7 +131,7 @@ def power_button_isa(cfg: PowerButtonConfig, perms: Context, should_exit: Event)
                     hold_dev.close()
                     hold_dev = None
                 logger.info(f"Waiting for steam to launch.")
-                while not is_steam_gamepad_running(perms):
+                while not is_steam_gamepad_running():
                     if should_exit.is_set():
                         return
                     sleep(STEAM_WAIT_DELAY)
@@ -146,12 +163,14 @@ def power_button_isa(cfg: PowerButtonConfig, perms: Context, should_exit: Event)
                 ev = press_dev.read_one()
                 if ev.type == B("EV_KEY") and ev.code == B("KEY_POWER") and ev.value:
                     logger.info("Executing short press.")
-                    issue_systemctl = not run_steam_shortpress(perms)
+                    issue_systemctl = not run_steam_shortpress()
+                    emit({"type": "special", "event": "pbtn_short"})
             elif hold_dev and fd == hold_dev.fd:
                 ev = hold_dev.read_one()
                 if ev.type == B("EV_KEY") and ev.code == cfg.hold_code and ev.value:
                     logger.info("Executing long press.")
-                    issue_systemctl = not run_steam_longpress(perms)
+                    issue_systemctl = not run_steam_longpress()
+                    emit({"type": "special", "event": "pbtn_long"})
 
             if issue_systemctl:
                 logger.error(
@@ -164,14 +183,16 @@ def power_button_isa(cfg: PowerButtonConfig, perms: Context, should_exit: Event)
         logger.error(f"Received exception, exitting:\n{e}")
 
 
-def power_button_timer(cfg: PowerButtonConfig, perms: Context, should_exit: Event):
+def power_button_timer(
+    cfg: PowerButtonConfig, should_exit: Event, emit
+):
     dev = None
     devs = []
     try:
         pressed_time = None
         while not should_exit.is_set():
             # Initial check for steam
-            if not is_steam_gamepad_running(perms):
+            if not is_steam_gamepad_running():
                 # Close devices
                 if devs:
                     for d in devs:
@@ -181,7 +202,7 @@ def power_button_timer(cfg: PowerButtonConfig, perms: Context, should_exit: Even
                         dev.close()
                         dev = None
                 logger.info(f"Waiting for steam to launch.")
-                while not is_steam_gamepad_running(perms):
+                while not is_steam_gamepad_running():
                     if should_exit.is_set():
                         return
                     sleep(STEAM_WAIT_DELAY)
@@ -230,10 +251,12 @@ def power_button_timer(cfg: PowerButtonConfig, perms: Context, should_exit: Even
             match press_type:
                 case "long_press":
                     logger.info("Executing long press.")
-                    issue_systemctl = not run_steam_longpress(perms)
+                    issue_systemctl = not run_steam_longpress()
+                    emit({"type": "special", "event": "pbtn_long"})
                 case "short_press":
                     logger.info("Executing short press.")
-                    issue_systemctl = not run_steam_shortpress(perms)
+                    issue_systemctl = not run_steam_shortpress()
+                    emit({"type": "special", "event": "pbtn_short"})
                 case "initial_press":
                     logger.info("Power button pressed down.")
                 case "release_without_press":
@@ -256,20 +279,22 @@ def power_button_timer(cfg: PowerButtonConfig, perms: Context, should_exit: Even
                 d.close()
 
 
-def power_button_multidev(cfg: PowerButtonConfig, perms: Context, should_exit: Event):
+def power_button_multidev(
+    cfg: PowerButtonConfig, should_exit: Event, emit
+):
     devs = []
     fds = []
     last_pressed = None
     try:
         while not should_exit.is_set():
             # Initial check for steam
-            if not is_steam_gamepad_running(perms):
+            if not is_steam_gamepad_running():
                 for d in devs:
                     d.close()
                 devs = []
                 fds = []
                 logger.info(f"Waiting for steam to launch.")
-                while not is_steam_gamepad_running(perms):
+                while not is_steam_gamepad_running():
                     if should_exit.is_set():
                         return
                     sleep(STEAM_WAIT_DELAY)
@@ -303,7 +328,8 @@ def power_button_multidev(cfg: PowerButtonConfig, perms: Context, should_exit: E
 
             if issue_power:
                 logger.info("Executing short press.")
-                issue_systemctl = not run_steam_shortpress(perms)
+                issue_systemctl = not run_steam_shortpress()
+                emit({"type": "special", "event": "pbtn_short"})
 
             if issue_systemctl:
                 logger.error(
